@@ -16,6 +16,7 @@ class AnomalyDetection:
         anomaly_sensitivity: float = 1,
         clustering_alg = KMeans(),
         quantize: bool = True,
+        quantize_type: str = "complex",
         eps: float = 0.1,
         verbose: bool = False
     ) -> None:
@@ -27,6 +28,8 @@ class AnomalyDetection:
             cluster_alg (sklearn.cluster, optional): clustering algorithm to use
                                                      (Default = KMeans())\n
             quantize (bool, optional): whether to quantize the data (Default = True)\n
+            quantize_type (str, optional): type of quantization to use ("complex" or "simple")
+                                           (Default = "complex")\n
             eps (float, optional): epsilon parameter for finding PFSAs (Default = 0.1)\n
             verbose (bool, optional): whether to print verbose output (Default = False)\n
 
@@ -35,6 +38,7 @@ class AnomalyDetection:
         self.anomaly_sensitivity = anomaly_sensitivity
         self.clustering_alg = clustering_alg
         self.quantize = quantize
+        self.quantize_type = quantize_type
         self.eps = eps
         self.verbose = verbose
         self.temp_dir = "zed_temp"
@@ -78,13 +82,14 @@ class AnomalyDetection:
         return self
 
 
-    def predict(self, X: Union[pd.DataFrame, str] = None) -> Union[bool, list[bool]]:
+    def predict(self, X: Union[pd.DataFrame, pd.Series, str] = None, *, clean: bool = True) -> Union[bool, list[bool]]:
         """ Predict whether a time series sequence is anomalous
 
         Args:
 
-            X (pd.DataFrame or str, optional): time series data to find anomalies, if None then
-                                               predicts on original data (Default = None)
+            X (pd.DataFrame or pd.Series or str, optional): time series data to find anomalies, if None then
+                                                            predicts on original data (Default = None)
+            clean (bool, optional): whether to remove temp files (Default = True)
 
         Returns:
 
@@ -92,24 +97,38 @@ class AnomalyDetection:
                                 output shape depends on input
         """
         seqfile = ""
+        num_predictions = 0
         # commonly want to find anomalies in original data
         if X is None:
             seqfile = self.data_file
+            num_predictions = len(self.clusters)
         else:
-            if type(X) is str and not self.quantize:
-                seqfile = X
-            else:
-                seqfile = RANDOM_NAME(path=self.temp_dir, clean=False)
-                self.__quantize(X).to_csv(
-                    seqfile,
-                    sep=" ",
-                    line_terminator=(" " if type(X) is pd.Series else "\n"),
-                    index=False,
-                    header=False
-                )
+            # if type(X) is str and not self.quantize:
+            #     seqfile = X
+            # else:
+            is_series = type(X) is pd.Series
+            seqfile = RANDOM_NAME(path=self.temp_dir, clean=True)
+            X_quantized = self.__quantize(X)
+            X_quantized.to_csv(
+                seqfile,
+                sep=" ",
+                line_terminator=(" " if is_series else "\n"),
+                index=False,
+                header=False
+            )
 
-        cluster_llk_vec = np.empty([len(self.cluster_PFSAs), len(self.clusters)], dtype=np.float64)
-        anomaly_vec = np.zeros(len(self.clusters), dtype=np.int8)
+            if is_series:
+                num_predictions = 1
+                # remove trailing space because it affects llk calculation
+                with open(seqfile, "r+") as f:
+                    line = next(f).rstrip()
+                    f.seek(0)
+                    f.write(line + "\n")
+            else:
+                num_predictions = X_quantized.shape[0]
+
+        cluster_llk_vec = np.empty([len(self.cluster_PFSAs), num_predictions], dtype=np.float64)
+        anomaly_vec = np.zeros(num_predictions, dtype=np.int8)
         for i in range(len(self.cluster_PFSAs)):
             curr_llks = Llk(seqfile=seqfile, pfsafile=self.cluster_PFSAs[i]).run()
             cluster_llk_vec[i] = curr_llks
@@ -125,7 +144,7 @@ class AnomalyDetection:
         if len(predictions) == 1:
             predictions = predictions[0]
 
-        if seqfile != self.data_file:
+        if seqfile != self.data_file and clean:
             os_remove(seqfile)
 
         return predictions
@@ -142,9 +161,9 @@ class AnomalyDetection:
                 print("\n")
 
 
-    def __quantize(self, X, quantize_type: str = "complex") -> pd.DataFrame:
+    def __quantize(self, X) -> pd.DataFrame:
         """ Quantize the data into finite alphabet """
-        if not self.quantize or quantize_type is None:
+        if not self.quantize or self.quantize_type is None:
             if type(X) is str:
                 return pd.read_csv(X, sep=" ", header=None, low_memory=False).dropna(how="all", axis=1)
             else:
@@ -154,16 +173,18 @@ class AnomalyDetection:
         # Quantizer() expects a DataFrame
         if type(X) is pd.Series:
             X = X.copy().to_frame().reset_index(drop=True).T
-        if quantize_type == "complex":
+        if self.quantize_type == "simple":
+            # basic differentiation
+            X = X.astype(float).diff(axis=1).fillna(0)
+            return X.apply(lambda row : row.apply(lambda n : 1 if n > 0 else 0), axis=1)
+        elif self.quantize_type == "complex":
             # use cythonized quantizer binary
             if not self._fitted:
                 self.quantizer = Quantizer(n_quantizations=1, epsilon=-1)
                 self.quantizer.fit(X)
             return pd.concat([quantized for quantized in self.quantizer.transform(X)], axis=1)
         else:
-            # basic differentiation
-            X = X.astype(float).diff(axis=1).fillna(0)
-            return X.apply(lambda row : row.apply(lambda n : 1 if n > 0 else 0), axis=1)
+            raise ValueError(f"Unknown quantize type: {self.quantize_type}. Choose \"simple\" or \"complex\".")
 
 
     def __calculate_dist_matrix(self, X) -> None:
@@ -192,13 +213,16 @@ class AnomalyDetection:
         n_clusters = len(set(self.clusters)) - (1 if -1 in self.clusters else 0)
         X["cluster"] = self.clusters
         cluster_files = []
-        for i in range(n_clusters):
-            if self.verbose:
-                print(f"Writing cluster {i + 1}/{n_clusters} to file...")
-            cluster_files.append(RANDOM_NAME(path=self.temp_dir))
-            X[X["cluster"] == i] \
-                .drop("cluster", axis=1) \
-                .to_csv(cluster_files[i], sep=" ", header=False, index=False, float_format="%g")
+        if (n_clusters == 1):
+            cluster_files.append(self.data_file)
+        else:
+            for i in range(n_clusters):
+                if self.verbose:
+                    print(f"Writing cluster {i + 1}/{n_clusters} to file...")
+                cluster_files.append(RANDOM_NAME(path=self.temp_dir))
+                X[X["cluster"] == i] \
+                    .drop("cluster", axis=1) \
+                    .to_csv(cluster_files[i], sep=" ", header=False, index=False, float_format="%g")
         self.cluster_files = cluster_files
 
 
